package impl

import (
	"context"
	"fmt"
	"net"
	"strings"

	"gitlab.mashroom.com/mashroom/backend/common/generic/metrics"

	"google.golang.org/grpc"

	"gitlab.mashroom.com/mashroom/backend/common/api/grpcerr"
	"gitlab.mashroom.com/mashroom/backend/common/api/health"
	"gitlab.mashroom.com/mashroom/backend/common/generic/log"
	twilio "gitlab.mashroom.com/mashroom/backend/twilio/api"
	"gitlab.mashroom.com/mashroom/backend/twilio/version"
)

type Server interface {
	twilio.TwilioServiceServer
	Serve(addr string)
	Background()
}

type serverImpl struct {
}

func (server *serverImpl) IVRFileHandler(ctx context.Context, request *twilio.IVRFileHandlerRequest) (*twilio.IVRFileHandlerResponse, error) {
	fmt.Printf("path = %v\n", request.File)
	file := ""
	if strings.Contains(request.File, "/") {
		path := strings.Split(request.File, "/")
		fmt.Printf("len(path) = %v\n", len(path))
		if len(path) != 5 {
			return nil, grpcerr.New(grpcerr.ProviderError, "file request bad")
		}
		file = path[len(path)-1]
	} else {
		file = request.File
	}
	fmt.Printf("file = %v\n", file)
	if file == "" {
		return nil, grpcerr.New(grpcerr.InvalidData, "file name empty")
	}

	data, err := VoiceFileHandlerImpl(file)
	if err != nil {
		return nil, err
	}
	fmt.Printf("data = %v\n", len(data))

	return &twilio.IVRFileHandlerResponse{
		File: file + ".mp3",
		Data: data,
		Mime: "audio/mpeg",
	}, nil
}

func (server *serverImpl) IVRCallbackHandler(ctx context.Context, request *twilio.IVRCallbackHandlerRequest) (*twilio.IVRCallbackHandlerResponse, error) {
	data, err := VoiceHandlerImpl(ctx, request.Data)
	if err != nil {
		return nil, err
	}

	return &twilio.IVRCallbackHandlerResponse{
		Data: data,
	}, nil
}

func (server *serverImpl) Background() {
	// background processes
}

func (server *serverImpl) Serve(addr string) {
	if listener, err := net.Listen("tcp", addr); err != nil {
		panic(err)
	} else {
		_ = log.SetHost("twilio")
		grpcServer := grpc.NewServer()

		twilio.RegisterTwilioServiceServer(grpcServer, server)

		healthServer := version.NewHealthServer()
		health.RegisterHealthServiceServer(grpcServer, healthServer)

		go func() {
			_ = metrics.ConfigureListenAndServe(grpcServer)
		}()

		log.Infoln("twilio started")
		if err := grpcServer.Serve(listener); err != nil {
			log.Errorln("gRPC error", err)
		}
	}
}

func NewServer() Server {
	server := &serverImpl{}
	go server.Background()
	return server
}
