package model

import (
	"context"
	"fmt"
	"time"

	"go.mongodb.org/mongo-driver/mongo"
	"go.mongodb.org/mongo-driver/mongo/options"

	"gitlab.mashroom.com/mashroom/backend/common/generic/env"
	"gitlab.mashroom.com/mashroom/backend/common/generic/log"
	"gitlab.mashroom.com/mashroom/backend/common/generic/migrations"
	migrationsMongo "gitlab.mashroom.com/mashroom/backend/common/generic/migrations/mongo"
)

// Config mongo repository config
type Config struct {
	URL        string
	Username   string
	Password   string
	Database   string
	AuthSource string
}

var (
	envMongoURL      = env.ResolveString("MONGO_URL", "mongodb:27017")
	envMongoUsername = env.ResolveString("MONGO_USERNAME", "twilio-data")
	envMongoPassword = env.ResolveString("MONGO_PASSWORD", "twilio-data")
	envMongoAuthDB   = env.ResolveString("MONGO_AUTH", "admin")
	envMongoDB       = env.ResolveString("MONGO_DATABASE", "twilio-data")
)

const (
	MongoTwilioCollection = "twilio"
)

var Client *mongo.Client
var DB *mongo.Database

var Twilios *mongo.Collection

func init() {
	go func() {
		config := &Config{
			URL:        envMongoURL,
			Username:   envMongoUsername,
			Password:   envMongoPassword,
			Database:   envMongoDB,
			AuthSource: envMongoAuthDB,
		}

		if c, err := mongo.NewClient(
			options.Client().ApplyURI(fmt.Sprintf("mongodb://%s", config.URL)).SetAuth(options.Credential{
				AuthSource:  config.AuthSource,
				Username:    config.Username,
				Password:    config.Password,
				PasswordSet: true,
			}),
		); err != nil {
			panic(err)
		} else {
			Client = c
		}

		ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
		defer cancel()
		if err := Client.Connect(ctx); err != nil {
			panic(err)
		}

		DB = Client.Database(config.Database)
		Twilios = DB.Collection(MongoTwilioCollection)

		err := migrations.MigrateSchema(migrationsMongo.WithInstance(ctx, DB),
			&migrationsMongo.Migration{
				Version: "v0",
				Up: func(ctx context.Context, db *mongo.Database, migrationID string) (int, error) {
					return -1, nil
				},
			},
		)

		if err != nil {
			log.WithError(err).Error()
		}
	}()
}
